==> ecological-engine-1.3.0-20121021.230420-75-sources.jar: org/gcube/dataanalysis/ecoengine/evaluation/DiscrepancyAnalysis.java <==
package org.gcube.dataanalysis.ecoengine.evaluation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.gcube.contentmanagement.graphtools.utils.MathFunctions;
import org.gcube.contentmanagement.lexicalmatcher.utils.AnalysisLogger;
import org.gcube.dataanalysis.ecoengine.datatypes.ColumnType;
import org.gcube.dataanalysis.ecoengine.datatypes.DatabaseType;
import org.gcube.dataanalysis.ecoengine.datatypes.InputTable;
import org.gcube.dataanalysis.ecoengine.datatypes.PrimitiveType;
import org.gcube.dataanalysis.ecoengine.datatypes.StatisticalType;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.PrimitiveTypes;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.TableTemplates;
import org.gcube.dataanalysis.ecoengine.interfaces.DataAnalysis;
import org.gcube.dataanalysis.ecoengine.utils.DatabaseFactory;
import org.gcube.dataanalysis.ecoengine.utils.DatabaseUtils;
import org.gcube.dataanalysis.ecoengine.utils.Operations;

public class DiscrepancyAnalysis extends DataAnalysis {

//	static String discrepancyQuery = "select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from %5$s as a inner join %6$s as b on a.%1$s=b.%2$s and (a.%3$s<>b.%4$s)";
//	static String discrepancyQuery = "select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from (select * from %5$s order by %1$s limit %7$s) as a inner join (select * from %6$s order by %2$s limit %7$s)  as b on a.%1$s=b.%2$s and (a.%3$s<>b.%4$s)";
	static String discrepancyQuery = "select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from " +
			"(select csquarecode,sum(%3$s) as %3$s from (select distinct * from %5$s order by %1$s limit %7$s) as aa group by %1$s) as a " +
			"left join " +
			"(select csquarecode,sum(%4$s) as %4$s from (select distinct * from %6$s order by %2$s limit %7$s) as aa group by %2$s) as b " +
			"on a.%1$s=b.%2$s and (a.%3$s<>b.%4$s)";
	
	static String getNumberOfElementsQuery = "select count(*) from %1$s";
	private static int minElements = 100;
	private static int maxElements = 30000;
	
	float threshold = 0.1f;
	String configPath = "./cfg/";
	
	List<Float> errors;
	double mean;
	double variance;
	int numberoferrors;
	int numberofvectors;
	float maxerror;
	String maxdiscrepancyPoint;
	private HashMap<String, String> output;

	@Override
	public List<StatisticalType> getInputParameters() {
		List<StatisticalType> parameters = new ArrayList<StatisticalType>();
		List<TableTemplates> templates = new ArrayList<TableTemplates>();
		templates.add(TableTemplates.HSPEC);
		templates.add(TableTemplates.TRAININGSET);
		templates.add(TableTemplates.TESTSET);
		InputTable p1 = new InputTable(templates,"FirstTable","First Table");
		InputTable p2 = new InputTable(templates,"SecondTable","Second Table");
		ColumnType p3 = new ColumnType("FirstTable", "FirstTableCsquareColumn", "the csquares column name in the first table", "csquarecode", false);
		ColumnType p4 = new ColumnType("SecondTable", "SecondTableCsquareColumn", "the csquares column name in the second table", "csquarecode", false);
		ColumnType p5 = new ColumnType("FirstTable", "FirstTableProbabilityColumn", "the probability column in the first table", "probability", false);
		ColumnType p13 = new ColumnType("SecondTable", "SecondTableProbabilityColumn", "the probability column in the second table", "probability", false);
		PrimitiveType p6 = new PrimitiveType(Float.class.getName(), null, PrimitiveTypes.NUMBER, "ComparisonThreshold","the comparison threshold","0.1");
		PrimitiveType p7 = new PrimitiveType(Integer.class.getName(), null, PrimitiveTypes.NUMBER, "MaxSamples","the comparison threshold","10000");
		
		parameters.add(p1);
		parameters.add(p2);
		parameters.add(p3);
		parameters.add(p4);
		parameters.add(p5);
		parameters.add(p13);
		parameters.add(p6);
		parameters.add(p7);
		
		DatabaseType.addDefaultDBPars(parameters);
		return parameters;
	}

	
	@Override
	public HashMap<String, String> analyze() throws Exception {

		String FirstTableCsquareColumn = config.getParam("FirstTableCsquareColumn");
		String SecondTableCsquareColumn = config.getParam("SecondTableCsquareColumn");
		String FirstTableProbabilityColumn = config.getParam("FirstTableProbabilityColumn");
		String SecondTableProbabilityColumn = config.getParam("SecondTableProbabilityColumn");
		String FirstTable = config.getParam("FirstTable");
		String SecondTable = config.getParam("SecondTable");
		String maxSamples =  config.getParam("MaxSamples");
		int maxCompElements = maxElements;
		if (maxSamples!=null && maxSamples.length()>0){
			int maxx = Integer.parseInt(maxSamples);
			maxCompElements = maxx!=0?maxx:Integer.MAX_VALUE;
		}
		
		List<Object> takeNPoints = DatabaseFactory.executeSQLQuery(String.format(getNumberOfElementsQuery, FirstTable), connection);
		List<Object> takeMPoints = DatabaseFactory.executeSQLQuery(String.format(getNumberOfElementsQuery, SecondTable), connection);
		int nPoints = Integer.parseInt(""+takeNPoints.get(0));
		int mPoints = Integer.parseInt(""+takeMPoints.get(0));
		numberofvectors = Math.max(nPoints, mPoints); 
		
		
		if (FirstTable.equals(SecondTable)){
			output = new HashMap<String, String>();
			output.put("MEAN", "0.0");
			output.put("VARIANCE", "0.0");
			output.put("NUMBER_OF_ERRORS", "0");
			output.put("NUMBER_OF_COMPARISONS", "" + numberofvectors);
			output.put("ACCURACY", "100.0");
			output.put("MAXIMUM_ERROR", "-");
			output.put("MAXIMUM_ERROR_POINT", "-");
			return output;
		}
		
//		String query = String.format(discrepancyQuery, FirstTableCsquareColumn, SecondTableCsquareColumn, FirstTableProbabilityColumn, SecondTableProbabilityColumn, FirstTable, SecondTable);
		List<Object> nelementsQ = DatabaseFactory.executeSQLQuery(DatabaseUtils.countElementsStatement(FirstTable),connection);
		int nelements = Integer.parseInt(""+nelementsQ.get(0));
		
		nelements = Math.min(Operations.calcNumOfRepresentativeElements(nelements, minElements),maxCompElements);
		
		AnalysisLogger.getLogger().trace("Number Of Elements to take: "+nelements);
		String query = String.format(discrepancyQuery, FirstTableCsquareColumn, SecondTableCsquareColumn, FirstTableProbabilityColumn, SecondTableProbabilityColumn, FirstTable, SecondTable,""+nelements);
		
				
		AnalysisLogger.getLogger().trace("Discrepancy Calculation - Query to perform :" + query);
		List<Object> takePoints = DatabaseFactory.executeSQLQuery(query, connection);

		super.processedRecords = 0;
		if (takePoints != null)
			super.processedRecords = takePoints.size();

		threshold = Float.parseFloat(config.getParam("ComparisonThreshold"));
		analyzeCompareList(takePoints);
		calcDiscrepancy();

		output = new HashMap<String, String>();
		output.put("MEAN", "" + MathFunctions.roundDecimal(mean,2));
		output.put("VARIANCE", "" + MathFunctions.roundDecimal(variance,2));
		output.put("NUMBER_OF_ERRORS", "" + numberoferrors);
		output.put("NUMBER_OF_COMPARISONS", "" + numberofvectors);

		float accuracy = 100;
		if (processedRecords>0)
			accuracy = (1 - (float) numberoferrors / (float) numberofvectors) * 100;
		
		
		output.put("ACCURACY", "" + MathFunctions.roundDecimal(accuracy,2));
		output.put("MAXIMUM_ERROR", "" + MathFunctions.roundDecimal(maxerror,2));
		output.put("MAXIMUM_ERROR_POINT", "" + maxdiscrepancyPoint);
		
		return output;
	}


	void calcDiscrepancy() {
		double[] err = new double[errors.size()];
		int i = 0;
		for (Float e : errors) {
			err[i] = e;
			i++;
		}

		mean = 0;
		variance = 0;

		if (err.length > 0) {
			mean = MathFunctions.mean(err);
			variance = com.rapidminer.tools.math.MathFunctions.variance(err, Double.NEGATIVE_INFINITY);
		}
	}

	public void analyzeCompareList(List<Object> points) {
		errors = new ArrayList<Float>();
		
		if (points != null) {
			maxerror = 0;
			for (Object vector : points) {
				Object[] elements = (Object[]) vector;
				String csquare = (String) elements[0];
				float probabilityPoint1 = 0;
				if (elements[2] != null)
					probabilityPoint1 = (Float) elements[2];
				float probabilityPoint2 = 0;
				if (elements[3] != null)
					probabilityPoint2 = (Float) elements[3];
				float discrepancy = Math.abs(probabilityPoint2 - probabilityPoint1);

				if (discrepancy > threshold) {
					errors.add(Math.abs(probabilityPoint2 - probabilityPoint1));
					numberoferrors++;
					if (discrepancy > maxerror) {
						maxerror = discrepancy;
						maxdiscrepancyPoint = csquare;
					}
				}
			}
		}

	}

		@Override
	public String getDescription() {
		return "Discrepancy Analysis bewteen two HSPEC distributions";
	}
}

==> ecological-engine-1.6.1-20130603.111043-63-sources.jar: org/gcube/dataanalysis/ecoengine/evaluation/DiscrepancyAnalysis.java <==
package org.gcube.dataanalysis.ecoengine.evaluation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

import org.gcube.contentmanagement.graphtools.utils.MathFunctions;
import org.gcube.contentmanagement.lexicalmatcher.utils.AnalysisLogger;
import org.gcube.dataanalysis.ecoengine.datatypes.ColumnType;
import org.gcube.dataanalysis.ecoengine.datatypes.DatabaseType;
import org.gcube.dataanalysis.ecoengine.datatypes.InputTable;
import org.gcube.dataanalysis.ecoengine.datatypes.PrimitiveType;
import org.gcube.dataanalysis.ecoengine.datatypes.StatisticalType;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.PrimitiveTypes;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.TableTemplates;
import org.gcube.dataanalysis.ecoengine.interfaces.DataAnalysis;
import org.gcube.dataanalysis.ecoengine.utils.DatabaseFactory;
import org.gcube.dataanalysis.ecoengine.utils.Operations;

public class DiscrepancyAnalysis extends DataAnalysis {

//	static String discrepancyQuery = "select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from %5$s as a inner join %6$s as b on a.%1$s=b.%2$s and (a.%3$s<>b.%4$s)";
//	static String discrepancyQuery = "select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from (select * from %5$s order by %1$s limit %7$s) as a inner join (select * from %6$s order by %2$s limit %7$s)  as b on a.%1$s=b.%2$s and (a.%3$s<>b.%4$s)";
	static String discrepancyQuery = "select * from (select distinct a.%1$s as csquareone,b.%2$s as csquaretwo,a.%3$s as firstprob,b.%4$s as secondprob from " +
			"(select %1$s , sum(%3$s) as %3$s from (select distinct * from %5$s order by %1$s limit %7$s) as aa group by %1$s) as a " +
			"left join " +
			"(select %2$s , sum(%4$s) as %4$s from (select distinct * from %6$s order by %2$s limit %7$s) as aa group by %2$s) as b " +
			"on a.%1$s=b.%2$s) as sel where firstprob<>secondprob";
	
	static String getNumberOfElementsQuery = "select count(*) from %1$s";
	private static int minElements = 100;
	private static int maxElements = 30000;
	
	float threshold = 0.1f;
	String configPath = "./cfg/";
	
	List<Float> errors;
	double mean;
	double variance;
	int numberoferrors;
	int numberofvectors;
	float maxerror;
	String maxdiscrepancyPoint;
	private LinkedHashMap<String, String> output;

	@Override
	public List<StatisticalType> getInputParameters() {
		List<StatisticalType> parameters = new ArrayList<StatisticalType>();
		List<TableTemplates> templates = new ArrayList<TableTemplates>();
		templates.add(TableTemplates.HSPEC);
		templates.add(TableTemplates.TRAININGSET);
		templates.add(TableTemplates.TESTSET);
		InputTable p1 = new InputTable(templates,"FirstTable","First Table");
		InputTable p2 = new InputTable(templates,"SecondTable","Second Table");
		ColumnType p3 = new ColumnType("FirstTable", "FirstTableCsquareColumn", "the csquares column name in the first table", "csquarecode", false);
		ColumnType p4 = new ColumnType("SecondTable", "SecondTableCsquareColumn", "the csquares column name in the second table", "csquarecode", false);
		ColumnType p5 = new ColumnType("FirstTable", "FirstTableProbabilityColumn", "the probability column in the first table", "probability", false);
		ColumnType p13 = new ColumnType("SecondTable", "SecondTableProbabilityColumn", "the probability column in the second table", "probability", false);
		PrimitiveType p6 = new PrimitiveType(Float.class.getName(), null, PrimitiveTypes.NUMBER, "ComparisonThreshold","the comparison threshold","0.1");
		PrimitiveType p7 = new PrimitiveType(Integer.class.getName(), null, PrimitiveTypes.NUMBER, "MaxSamples","the comparison threshold","10000");
		
		parameters.add(p1);
		parameters.add(p2);
		parameters.add(p3);
		parameters.add(p4);
		parameters.add(p5);
		parameters.add(p13);
		parameters.add(p6);
		parameters.add(p7);
		
		DatabaseType.addDefaultDBPars(parameters);
		return parameters;
	}

	
	@Override
	public LinkedHashMap<String, String> analyze() throws Exception {

		String FirstTableCsquareColumn = config.getParam("FirstTableCsquareColumn");
		String SecondTableCsquareColumn = config.getParam("SecondTableCsquareColumn");
		String FirstTableProbabilityColumn = config.getParam("FirstTableProbabilityColumn");
		String SecondTableProbabilityColumn = config.getParam("SecondTableProbabilityColumn");
		String FirstTable = config.getParam("FirstTable");
		String SecondTable = config.getParam("SecondTable");
		String maxSamples =  config.getParam("MaxSamples");
		int maxCompElements = maxElements;
		if (maxSamples!=null && maxSamples.length()>0){
			int maxx = Integer.parseInt(maxSamples);
			maxCompElements = maxx!=0?maxx:Integer.MAX_VALUE;
		}
		
		List<Object> takeNPoints = DatabaseFactory.executeSQLQuery(String.format(getNumberOfElementsQuery, FirstTable), connection);
		List<Object> takeMPoints = DatabaseFactory.executeSQLQuery(String.format(getNumberOfElementsQuery, SecondTable), connection);
		int nPoints = Integer.parseInt(""+takeNPoints.get(0));
		int mPoints = Integer.parseInt(""+takeMPoints.get(0));
		numberofvectors = Math.max(nPoints, mPoints); 
		
		
		if (FirstTable.equals(SecondTable)){
			output = new LinkedHashMap<String, String>();
			output.put("MEAN", "0.0");
			output.put("VARIANCE", "0.0");
			output.put("NUMBER_OF_ERRORS", "0");
			output.put("NUMBER_OF_COMPARISONS", "" + numberofvectors);
			output.put("ACCURACY", "100.0");
			output.put("MAXIMUM_ERROR", "-");
			output.put("MAXIMUM_ERROR_POINT", "-");
			return output;
		}
		
//		String query = String.format(discrepancyQuery, FirstTableCsquareColumn, SecondTableCsquareColumn, FirstTableProbabilityColumn, SecondTableProbabilityColumn, FirstTable, SecondTable);
//		List<Object> nelementsQ = DatabaseFactory.executeSQLQuery(DatabaseUtils.countElementsStatement(FirstTable),connection);
//		int nelements = Integer.parseInt(""+nelementsQ.get(0));
		
//		int nelements = Math.min(Operations.calcNumOfRepresentativeElements(nPoints, minElements),maxCompElements);
		int nelements = nPoints;
		
		AnalysisLogger.getLogger().trace("Number Of Elements to take: "+nelements);
		String query = String.format(discrepancyQuery, FirstTableCsquareColumn, SecondTableCsquareColumn, FirstTableProbabilityColumn, SecondTableProbabilityColumn, FirstTable, SecondTable,""+nelements);
		
		AnalysisLogger.getLogger().debug("Discrepancy Calculation - Query to perform :" + query);
		List<Object> takePoints = DatabaseFactory.executeSQLQuery(query, connection);

		super.processedRecords = 0;
		if (takePoints != null)
			super.processedRecords = takePoints.size();

		threshold = Float.parseFloat(config.getParam("ComparisonThreshold"));
		analyzeCompareList(takePoints);
		calcDiscrepancy();

		output = new LinkedHashMap<String, String>();
		output.put("MEAN", "" + MathFunctions.roundDecimal(mean,2));
		output.put("VARIANCE", "" + MathFunctions.roundDecimal(variance,2));
		output.put("NUMBER_OF_ERRORS", "" + numberoferrors);
		output.put("NUMBER_OF_COMPARISONS", "" + nelements);

		float accuracy = 100;
		if (processedRecords>0)
			accuracy = (1 - (float) numberoferrors / (float) nelements) * 100;
		
		
		output.put("ACCURACY", "" + MathFunctions.roundDecimal(accuracy,2));
		output.put("MAXIMUM_ERROR", "" + MathFunctions.roundDecimal(maxerror,2));
		output.put("MAXIMUM_ERROR_POINT", "" + maxdiscrepancyPoint);
		if (numLower>numHigher)
			output.put("TREND", "CONTRACTION");
		if (numLower<numHigher)
			output.put("TREND", "EXPANSION");
		else 
			output.put("TREND", "STATIONARY");
		
		return output;
	}


	void calcDiscrepancy() {
		double[] err = new double[errors.size()];
		int i = 0;
		for (Float e : errors) {
			err[i] = e;
			i++;
		}

		mean = 0;
		variance = 0;

		if (err.length > 0) {
			mean = MathFunctions.mean(err);
			variance = com.rapidminer.tools.math.MathFunctions.variance(err, Double.NEGATIVE_INFINITY);
		}
	}

	
	long numHigher = 0;
	long numLower = 0;
	public void analyzeCompareList(List<Object> points) {
		errors = new ArrayList<Float>();
		
		if (points != null) {
			maxerror = 0;
			for (Object vector : points) {
				Object[] elements = (Object[]) vector;
				String csquare = (String) elements[0];
				float probabilityPoint1 = 0;
				if (elements[2] != null)
					probabilityPoint1 = (Float) elements[2];
				float probabilityPoint2 = 0;
				if (elements[3] != null)
					probabilityPoint2 = (Float) elements[3];
				float discrepancy = Math.abs(probabilityPoint2 - probabilityPoint1);
				
				if (discrepancy > threshold) {
					errors.add(Math.abs(probabilityPoint2 - probabilityPoint1));
					numberoferrors++;
					if (discrepancy > maxerror) {
						maxerror = discrepancy;
						maxdiscrepancyPoint = csquare;
					}
					if (probabilityPoint2>probabilityPoint1)
						numHigher++;
					if (probabilityPoint2<probabilityPoint1)
						numLower++;
				}
			}
		}

	}

		@Override
	public String getDescription() {
		return "Discrepancy Analysis bewteen two HSPEC distributions";
	}
}
